(function($){ // To-do: remove after prototype removed
    var SmtpSettings = function(data){
        this.constructor(data);
    };

    SmtpSettings.prototype = {
        constructor: function(data){
            this.isWcloud = data.is_wcloud;
            this._data = data.smtpProfile;
            this._pbxName = data.pbxName;

            this.mainForm = $('#smtp_settings_form');
            this.buttonSave = this.mainForm.find('#smtp_settings_save').click($.proxy(this.save, this));

            this.testForm = $('#smtp_test_form');
            this.testLog = $('#smtp_test_output_div');
            this.buttonTest = this.testForm.find('#smtp_settings_test').click($.proxy(this.testRun, this));

            this.mainForm.checkForm({
                rules: [
                    'required,smtp_port,' + WMS._t('rules.CannotBeEmpty'),
                    'digits_only,smtp_port,' + WMS._t('rules.enterNumber'),
                    'range=1-65535,smtp_port,' + WMS._t('rules.enterValidValue') + ' 1-65535',
                    'valid_email,smtp_from,' + WMS._t('rules.enterValidValue')
                ]
            });
            this.testForm.checkForm({
                rules: [
                    'required,smtp_test_address,' + WMS._t('rules.CannotBeEmpty'),
                    'valid_email,smtp_test_address,' + WMS._t('rules.enterValidValue')
                ]
            });
            this.mainForm.find('#smtp_password').passwordToggler({
                textShowPassword: WMS._t('strings.Show'),
                textHidePassword: WMS._t('strings.Hide')
            });

            if (this.isWcloud){
                this.mainForm.find('#smtp_is_default_settings').change($.proxy(this.checkEditMode, this));
            } else {
                // Do not send to server this option at all
                this.mainForm.find('#smtp_is_default_settings').closest('tr').remove();
            }
            $(window).bind('beforeunload', $.proxy(function(){
                if (this.isDirty()){
                    return WMS._t('prefs.are_you_sure_want_to_exit_without_save');
                }
            }, this));
        },
        load: function(){
            if (this._data){
                this.setData(this._data);
            }
            this.checkEditMode();
            return this;
        },
        validate: function(){
            return this.mainForm.checkForm('validate');
        },
        validateTest: function(){
            if (this.isDirty()){
                return setError('error', WMS._t('prefs.save_settings_before'), 5000);
            }
            return this.testForm.checkForm('validate');
        },
        getData: function(){
            var data = {};
            this.mainForm.find('input,select').not('[type=button]').each(function(item) {
                var id = $(this).attr('id');
                if (!id) {
                    return;
                }
                // Remove form prefix to retrive real name
                var name = id.split('smtp_')[1];
                var value = $(this).val();
                if (name){
                    if (['tls', 'tls_starttls', 'is_default_settings'].indexOf(name) > -1){
                        value = $(this).is(':checked') ? 'on' : 'off';
                    }
                    data[name] = value;
                }
            });
            return data;
        },
        setData: function(data){
            $.each(data, $.proxy(function(name, value){
                if (['tls', 'tls_starttls', 'is_default_settings'].indexOf(name) > -1){
                    this.mainForm.find('#smtp_' + name).prop('checked', value == 'on');
                } else {
                    if (name == 'password'){
                        this.mainForm.find('#smtp_' + name).passwordToggler('val', value);
                    } else {
                        this.mainForm.find('#smtp_' + name).val(value);
                    }
                }
            }, this));
        },
        isDirty: function(){
            var result = false;
            $.each(this.getData(), $.proxy(function(name, value){
                if (this._data[name] != value){
                    result = true;
                    return false;
                }
            }, this));
            return result;
        },
        checkEditMode: function(data){
            if (this.isWcloud && this.mainForm.find('#smtp_is_default_settings').is(':checked')){
                this.mainForm.find('input,select').not('input[type=button]').not('#smtp_is_default_settings').prop('disabled', true);
                this.mainForm.find('#smtp_password').passwordToggler('hide');
            } else {
                this.mainForm.find('input,select').not('input[type=button]').not('#smtp_is_default_settings').prop('disabled', false);
                this.mainForm.find('#smtp_password').passwordToggler('show');
            }
        },
        save: function(){
            if (!this.validate()){
                return;
            }
            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/smtp/',
                type: 'POST',
                dataType: 'json',
                data: {
                    data: this.getData(),
                    _METHOD: 'PUT'
                },
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.result && response.result.account){
                        this._data = response.result;
                        this.load();
                    } else {
                        // Store to detect dirty flag in future
                        this._data = this.getData();
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    this.buttonTest.prop('disabled', false);
                    setError('error', this.parseResponseError(jqXHR));
                }, this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        testRun: function(){
            if (!this.validateTest()) {
                return;
            }
            this.testLog.text('');
            this.buttonTest.prop('disabled', true);
            $.ajax({
                url: 'api/v1/PBX/settings/smtp/test/',
                type: 'POST',
                dataType: 'json',
                data: {
                    data: {
                        from: this.getData().from,
                        mailto: this.testForm.find('#smtp_test_address').val(),
                        body: WMS._t('prefs.smtp_testmailbody', this._pbxName)
                    }
                },
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.result && response.result.queued){
                        this.testStatus(response.result.testId);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    this.buttonTest.prop('disabled', false);
                    setError('error', this.parseResponseError(jqXHR));
                }, this)
            });
        },
        testStatus: function(testId){
            $.ajax({
                url: 'api/v1/PBX/settings/smtp/test/status/' + testId,
                type: 'GET',
                dataType: 'json',
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.result){
                        if (response.result.queued){
                            setTimeout($.proxy(this.testStatus, this, response.result.testId), 500);
                        } else {
                            this.buttonTest.prop('disabled', false);
                        }
                        this.setTestLog(response.result.content);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    this.buttonTest.prop('disabled', false);
                    setError('error', this.parseResponseError(jqXHR));
                }, this)
            });
        },
        parseResponseError: function(jqXHR){
            var error = '';
            if(jqXHR && jqXHR.responseText && jqXHR.responseText.indexOf('{') === 0){
                var response = jQuery.parseJSON(jqXHR.responseText);
                if(response && response.reason){
                    error = String(response.reason);
                }
            }
            return error || 'Unknown error';
        },
        setTestLog: function(text){
            // Encode html entities and replace line breaking with br tag
            text = $('<div>').text(text || '').html().replace(/\r\n/g, '<br/>').replace(/[\r\n]/g, '<br/>');
            // Format for key words
            text = text.replace(/(using)/g, '<b>$1</b>').replace(/(loaded)/g, '<b>$1</b>');
            // Format for errors
            text = text.replace(/(msmtp:.+?<br\/>)/g, "<b class='log_mark_out_error'>$1</b>");

            // Green white
            text = text.replace(/--&gt;(.+?)<br\/>/g, "<div class='log_mark_send_smtp'><i class='fa fa-arrow-right fa-1'></i>$1</div>");
            text = text.replace(/&lt;--(.+?)<br\/>/g, "<div class='log_mark_receive_smtp'><i class='fa fa-arrow-left fa-1'></i>$1</div>");

            this.testLog.html(text);
            this.testLog.animate({scrollTop: 99999});
            return text;
        }
    };

    $(document).ready(function() {
        window.sm = new SmtpSettings(pageData).load();
    });
})(jQuery);
